c  VERSION 1.1  (NLSP version)   3/1/94
c----------------------------------------------------------------------
c                  ===================
c                     program NLSPC
c                  ===================
c
c     MAIN PROGRAM FOR NON-LINEAR LEAST SQUARES FIT OF 2D-SPECTRA
c
c     This program reads in parameters and data for a nonlinear least-
c     squares fit using an EPRCGF-family slow-motional 2D spectral
c     calculation.
c
c     ** Needs graphical interface **
c
c     Includes:
c        nlsdim.inc
c        nlsnam.inc
c        eprprm.inc
c        expdat.inc
c        parcom.inc
c        prmeqv.inc
c        lmcom.inc 
c        stdio.inc
c        iterat.inc
c
c######################################################################
c
      program nlspc
c
      implicit none
c
      include 'nlsdim.inc'
      include 'nlsnam.inc'
      include 'eprprm.inc'
      include 'expdat.inc'
      include 'parcom.inc'
      include 'prmeqv.inc'
      include 'lmcom.inc'
      include 'stdio.inc'
      include 'iterat.inc'
c
      integer i,iflg,ioerr,j,lth
      logical fexist
      character line*80,token*30,scratch*30,fileID*30,chr*2
c
      logical getlin
      external getlin
c
c######################################################################
c
c  VS-FORTRAN kludges #################
c      call xuflow(0)
c      call errset(209,0,-1,1)
c      call errset(215,0,-1,1)
c #####################################
c
      write (luttyo,1000)
      write (luttyo,1010)
c
c----------------------------------------------------------------------
c    Initializations
c----------------------------------------------------------------------
      ihltcmd=0
      ihltfit=0
c
      nfiles=0
      lucmd=luttyi
      luout=luttyo
      luecho=luttyo
      nspc=0
      nptot=0
      nptotcw=0
      nprm=0
      nser=0
      nsparm=0
      negegv=0
      ldelta=1
      kdelta=1
      setbas=.false.
      diaflg=.false.
c
c----------------------------------------
c     Initialize parameter arrays
c----------------------------------------
      do 22 i=1,nvprm
         fparm(i)=0.0d0
         ixx(i)=0
 22   continue
      fparm(ICGTOL)=1.0D-4
      fparm(ISHIFTR)=0.1D0
c
      do 23 i=1,niprm
         iparm(i)=0
 23   continue
c
      iparm(INORT)=1
      iparm(ILEMX)=8
      iparm(ILEMX+1)=5
      iparm(ILEMX+2)=4
      iparm(ILEMX+3)=2
      iparm(ILEMX+4)=2
      iparm(IITYPE)=1
      iparm(ISPTST)=0
c
      aflag=.false.
      gflag=.false.
      rotflg=.false.
      potflg=.false.
      uniflg=.false.
c
c  -- Set initial values for convergence criteria
c
      xtol=1.0d-6
      ftol=1.0d-6
      gtol=1.0d-8
      factor=1.0d2
      maxev=100
      maxitr=5
      maxfit=1
      itrace=0
      idebug=0
c
c  -- Set initial values for line search parameters
c
      sstep=0.05d0
      stol=1.0d-3
c
c----------------------------------------------------------------------
c     Open pipe loop for windows update
c----------------------------------------------------------------------
c      call openpipe()
c
c   ***************                                 ***************
c   ***************  Beginning of MAIN COMMAND LOOP ***************
c   ***************                                 ***************
c
c----------------------------------------------------------------------
c     Get next command from input stream (skip comments)
c----------------------------------------------------------------------
c
 25   if ((ihltcmd.ne.0).and.(nfiles.gt.0)) then
         if (luout.ne.luttyo) write(luttyo,1004)
         write(luout,1004)
         do 27 i=1,nfiles
            close (ludisk+i)
 27      continue
         lucmd=luttyi
         call uncatchc(ihltcmd)
      end if
c
      if (getlin(line)) then
c
c######################################################################
c
         call gettkn(line,token,lth)
         call touppr(token,lth)
         if (lth.eq.0 .or. token.eq.'C' .or. token.eq.'/*') go to 25
c
c----------------------------------------------------------------------
c        AXIAL command
c----------------------------------------------------------------------
         if (token.eq.'AXIAL') then
            call convtc(line,3)
c
c----------------------------------------------------------------------
c        BASIS command
c----------------------------------------------------------------------
         else if (token.eq.'BASIS') then
c
            do 28 i=1,nfprm
               fepr(i)=fparm(i)
 28         continue
            do 29 i=1,niprm
               iepr(i)=iparm(i)
 29         continue
c
            if (setbas) setbas=.false.
c
            call basis(line,luout)
c
c----------------------------------------------------------------------
c        CARTESIAN command
c----------------------------------------------------------------------
         else if (token.eq.'CARTESIAN' .or. token.eq.'CART') then
            call convtc(line,1)
c
c----------------------------------------------------------------------
c        DATA command
c----------------------------------------------------------------------
         else if (token.eq.'DATA') then
            call datac(line)
c
c----------------------------------------------------------------------
c        DEBUG command: open debug file identification
c----------------------------------------------------------------------
         else if (token.eq.'DEBUG') then
            idebug=1
            open(ludeb,file=dbname(:lthfnm),status='unknown',
     #           access='sequential',form='formatted',iostat=ioerr)
c
            ievec=0
            call gettkn(line,token,lth)
            call touppr(token,lth)
	    if (token.eq.'VECTOR') ievec=1
c
c----------------------------------------------------------------------
c        ECHO command
c----------------------------------------------------------------------
         else if (token.eq.'ECHO') then
            call gettkn(line,token,lth)
            if (lth.gt.0) then
               scratch=token
            else
               scratch=' '
            end if
            call touppr(scratch,3)
            if (scratch.eq.'ON') then
               luecho=luttyo
            else if (scratch.eq.'OFF') then
               luecho=0
            else
               call ungett(token,lth,line)
               write(luttyo,1060) line
            end if
c
c----------------------------------------------------------------------
c        FIT command
c----------------------------------------------------------------------
         else if (token.eq.'FIT') then
            call fitc(line)
c
c----------------------------------------------------------------------
c        FIX command (alias REMOVE)
c----------------------------------------------------------------------
         else if (token.eq.'FIX' .or. token.eq.'REMOVE') then
            call fixc(line)
c
c----------------------------------------------------------------------
c        HELP command
c----------------------------------------------------------------------
         else if (token.eq.'HELP') then
            call helpc(line)
c
c----------------------------------------------------------------------
c        LET command
c----------------------------------------------------------------------
         else if (token.eq.'LET') then
            call letc(line)
c
c----------------------------------------------------------------------
c        LOG command: Set log file identification
c----------------------------------------------------------------------
         else if (token.eq.'LOG') then
            call gettkn(line,fileID,lth)
            if (lth.eq.0) then
               write (luttyo,1020)
c
            else if (fileID.eq.'END' .or. fileID.eq.'end') then
               if (luout.eq.luttyo) then
                  write (luttyo,1021)
               else
                  close(lulog)
                  luout=luttyo
               end if
c
            else
               call setfil( fileID )
c
               open(lulog,file=lgname(:lthfnm),status='unknown',
     #         access='sequential',form='formatted',iostat=ioerr)
c
               inquire(file=lgname(:lthfnm),exist=fexist)
               if (fexist) then
c                                   * append the log file
 30               read (lulog,'(a2)',end=32) chr
                  go to 30
               end if
 32            luout=lulog
            end if              
c
c----------------------------------------------------------------------
c        PARMS command
c----------------------------------------------------------------------
         else if (token.eq.'PARMS') then
            call parc(line)
c
c----------------------------------------------------------------------
c        QUIT command (alias EXIT)
c----------------------------------------------------------------------
         else if (token.eq.'QUIT'.or.token.eq.'EXIT') then
            goto 9999
c
c----------------------------------------------------------------------
c        READ command (alias CALL)
c        open a new input file and set I/O units appropriately
c----------------------------------------------------------------------
c
         else if (token.eq.'READ' .or. token.eq.'CALL') then
c
c       --- get filename
c
            call gettkn(line,fileID,lth)
c
            if (lth.ne.0) then
c
c       --- open input file if possible
c
               if (nfiles.ge.mxfile) then
                  write (luttyo,1050) fileID(:lth),mxfile
               else
                  nfiles=nfiles+1
                  lucmd=ludisk+nfiles
                  inquire(file=fileID(:lth),exist=fexist)
                  if (fexist) open(lucmd,file=fileID(:lth),
     #                    status='old',access='sequential',
     #                    form='formatted',iostat=ioerr)
                  if ((.not.fexist) .or. ioerr.ne.0) then
c                                               *** open error
                     write (luttyo,1030) fileID(:lth)
                     nfiles=nfiles-1
                     if (nfiles.eq.0) then
                        lucmd=luttyi
                     else
                        lucmd=lucmd-1
                     end if
c
                  else
                     files(nfiles)=fileID
c
                     call catchc( ihltcmd )
c                     
                  end if
               end if
c                              *** File identification not specified
            else
               write (luttyo,1020) 
            end if
c
c----------------------------------------------------------------------
c        SEARCH command
c----------------------------------------------------------------------
         else if (token.eq.'SEARCH') then
            call srchc(line)
c
c----------------------------------------------------------------------
c        SERIES command
c----------------------------------------------------------------------
         else if (token.eq.'SERIES') then
            call series(line)
c
c----------------------------------------------------------------------
c        SPHERICAL command
c----------------------------------------------------------------------
         else if (token.eq.'SPHERICAL' .or. token.eq.'SPHER') then
            call convtc(line,2)
c
c----------------------------------------------------------------------
c        STATUS command 
c----------------------------------------------------------------------
         else if (token.eq.'STATUS') then
            call statc(line,luttyo)
c
c----------------------------------------------------------------------
c        UNIFORM command
c----------------------------------------------------------------------
         else if (token.eq.'UNIFORM') then
            uniflg=.true.
c
c----------------------------------------------------------------------
c        VARY command
c----------------------------------------------------------------------
         else if (token.eq.'VARY') then
            call varyc(line)
c
c----------------------------------------------------------------------
c        Unknown command  
c----------------------------------------------------------------------
         else
            write(luttyo,1040) token(:lth)
         end if
c
c----------------------------------------------------------------------
c    No more lines (getlin returned .false.)
c    Close current input unit; if there are no open files, stop program
c----------------------------------------------------------------------
c
      else
         if (nfiles.eq.0) then
            write(luttyo,1000)
            stop 'end of program NLSPC'
         else
            close(lucmd)
            nfiles=nfiles-1
            if (nfiles.eq.0) then
               lucmd=luttyi
               call uncatchc(ihltcmd)
            else
               lucmd=lucmd-1
            end if
         end if
      end if
      go to 25
c
c----------------------------------------------------------------------
c     Exit program
c----------------------------------------------------------------------
c
 9999 continue
c
c######################################################################
c Shut all windows before exiting
c
c      call shtwndws()
c      call shtwndwspdt()
c      call closepipe()
c######################################################################
c
      stop
c
c## format statements ###############################################
c
 1000 format(//,2x,70('#'),//)
 1004 format(/20x,'*** MACRO execution halted by user ***')
 1010 format(25x,'PROGRAM : NLSPC',/,25x,'---------------',//)
 1020 format('*** File identification must be specified ***'/)
 1021 format('*** Log file is not open ***')
 1030 format('*** Error opening or reading file ''',a,''' ***'/)
 1040 format('*** Unknown command : ''',a,''' ***')
 1050 format('*** Cannot open ''',a,''': more than',i2,
     #       ' read files ***')
 1060 format(a)
      end
