c  VERSION 1.0  (NLS version)   4/11/93
c*********************************************************************
c
c                       =================
c                       SUBROUTINE CONVFT
c                       =================
c
c     This routine performs convolution with Gaussian inhomogeneous
c     broadening and subsequent FT on comlex 2D time-domain data to
c     generate frequency domain spectrum.  It is intended for use
c     with nonlinear least-squares applications.
c
c     On Entry :
c        xspec :  complex 2D spectrum in time domain
c
c     On Exit  :
c        spec  :  magnitude spectrum in freq. domain
c
c     Includes :
c               nlsdim.inc
c               eprprm.inc
c               wkspcm.inc
c               physcn.inc
c
c     Uses :
c               fft.f
c               switch.f
c
c*********************************************************************
c
      subroutine convft(xspec,spec,npt1,npt2)
c
      implicit none
c
      include 'nlsdim.inc'
      include 'eprprm.inc'
      include 'wkspcm.inc'
      include 'physcn.inc'
c     
      double precision zero,gtf,cfact,twopi
      parameter (zero=0.0d0)
c
      integer i,j,npt1,npt2
      double precision gib0,lib0,tdabs,expnt,gconv,t1,t2,
     #        linit1,lstept1,linit2,lstept2
c
      double precision spec(npt1,npt2)

      complex*16 xspec(npt1,npt2)
c
c#####################################################################
c
      twopi=8.0d0*datan(1.0d0)
      gtf=g0*betae/(hbar*twopi)*1.0d-6
      cfact=twopi*gtf
c
      linit1=init1*1.0d-3
      lstept1=stept1*1.0d-3
      if (iexp.ge.1) then
         linit2=init2*1.0d-3
         lstept2=stept2*1.0d-3
      end if
c
      gib0=dabs(gib*cfact/twopi)
      lib0=dabs(lib*cfact)
c
c---------------------------------------------------------------------
c     Convolution with Gaussian and/or Lorentzian inhomogeneous broad.
c---------------------------------------------------------------------
c
 400  if ((gib0.gt.1.0d-13).or.(lib0.gt.1.0d-13)) then
c
         gib0=twopi*twopi*gib0*gib0/2.0d0
         do 410 i=1,npt1
            t1=linit1+(i-1)*lstept1
c                                     ** FID **             
            if (iexp.eq.0) then
               c2wsp1(i,1)=xspec(i,1)*dexp(-gib0*t1*t1)*dexp(-lib0*t1)
               go to 410
            end if
c
            if ((iexp.eq.2).or.(iexp.eq.4).or.(iexp.eq.5)) t1=zero
            do 420 j=1,npt2             
               t2=linit2+(j-1)*lstept2
               tdabs=dabs(t2+icomb*t1)
               expnt=gib0*tdabs*tdabs
               gconv=zero
               if (expnt.le.5.0d1)  gconv=dexp(-expnt)
               c2wsp1(i,j)=xspec(i,j)*gconv*dexp(-lib0*tdabs)
 420        continue
 410     continue
c
      else
         do 430 j=1,npt2
         do 430 i=1,npt1
            c2wsp1(i,j)=xspec(i,j)
 430     continue
      end if
c
c---------------------------------------------------------------------
c     Frequency spectrum by Fourier transform
c---------------------------------------------------------------------
c
      if (iexp.ge.1) then
c                                 **  along t2
         do 500 i=1,npt1
            do 510 j=1,npt2
 510           cwsp3(j)=c2wsp1(i,j)
            call fft(cwsp3,npt2)
            call switch(cwsp3,npt2)
            do 520 j=1,npt2
 520           c2wsp1(i,j)=cwsp3(j)
 500     continue
c                                 **  along t1
         do 550 j=1,npt2
            do 560 i=1,npt1
 560           cwsp3(i)=c2wsp1(i,j)
            call fft(cwsp3,npt1)
            call switch(cwsp3,npt1)
            do 570 i=1,npt1
 570           spec(i,j)=cdabs(cwsp3(i))
 550     continue
c
      else
c
         do 580 i=1,npt1
 580        cwsp3(i)=c2wsp1(i,1)
         call fft(cwsp3,npt1)
         call switch(cwsp3,npt1)
         do 590 i=1,npt1
 590        spec(i,1)=cdabs(cwsp3(i))
c
      end if
c
      return
      end
