/*************************************************************************
 Eigenvector output routines designed to be called from FORTRAN
   program EPRPS.

   Calling sequence:
       iret = iewopen( name )  -- open a file for binary write
            "name" is a character variable containing the output filename
            "iret" is 0 for successful open, otherwise there was an error

       iret = ieropen( name )  -- open a file for binary read
            "name" is a character variable containing the output filename
            "iret" is 0 for successful open, otherwise there was an error
            
       iret = iewrite( vec, n ) -- write a complex vector to output
            "vec" is a complex*16 vector
            "n" is the number of elements in vec
            "iret" is the number of elements actually written

       iret = ieread( vec, n ) -- write a complex vector to output
            "vec" is a complex*16 vector
            "n" is the number of elements in vec
            "iret" returns the number of elements successfully read

       iret = ieclose() -- close an input or output file
            Returns 0 for succesful close

************************************************************************* */

#include <stdio.h>

FILE *iofile;

struct complex {
  double real,imag;
  };

/*   trim **************************************************  
  Remove trailing blanks from a string passed from FORTRAN
*********************************************************** */

char *trim( char *str, long int *lth )
{
int i=0;
   while ((str[i++] != ' ') && (i <= *lth)); 
   str[i-1] = 0;
   return str;
}

/**** iewopen ***************************************************
  Open a file for output
**************************************************************** */

long int iewopen( char *filename, long int *lth )
{
   iofile = fopen( trim(filename,lth), "w" );
   return !iofile;
}

/**** ieropen ***************************************************
  Open a file for input
**************************************************************** */
long int ieropen( char *filename, long int *lth )
{
   iofile = fopen( trim(filename,lth), "r" );
   return !iofile;
}

/**** iewrite **************************************************
  Write a complex vector to the output file
**************************************************************** */

long int iewrite( struct complex vec[], long int *n )
{
return (long int) fwrite( vec, sizeof( *vec ), *n, iofile );
}

/**** ieread **************************************************
  Read a complex vector from the input file
**************************************************************** */

long int ieread( struct complex vec[], long int *n )
{
   return (long int) fread( vec, sizeof( *vec ), *n, iofile );
}

/**** ieclose **************************************************
  Close the output file
**************************************************************** */

long int ieclose()
{
return fclose( iofile );
}
