c  VERSION 1.1  (NLSPC version)   3/1/94
c*********************************************************************
c
c                       ==================
c                       SUBROUTINE : EVCGF
c                       ==================
c
c     Subroutine version of EPRCGF program. (intended for NLSP)
c
c     It calculates the eigenvalues and eigenvectors for the general 
c     complex matrix using Conjugate gradient algorithm.
c
c     On Entry :
c
c        Parameters are passed through the common block /eprprm/.
c        Pulse propagator is passed through the common block /indexf/.
c        Off-diagonal basis vectors are passed through the common
c                                                      block /indexf/.
c
c     On Exit  :
c
c        nev   : number of important eigenvalues
c        eval  : important eigenvalues
c        evec  : important eigenvectors
c        ierr  : error flag
c
c     Includes :
c               nlsdim.inc
c               stdio.inc
c               eprprm.inc
c               parcom.inc
c               eprmat.inc
c               indexf.inc
c               wkspcm.inc
c
c     Uses :
c               matrxo.f
c               matrxd.f
c               cscg.f
c               csval.f
c               csvec.f
c
c*********************************************************************
c
      subroutine evcgf(psval,x,nev,eval,evec,ierr)
c
      implicit none
c
      include 'nlsdim.inc'
      include 'stdio.inc'
      include 'eprprm.inc'
      include 'parcom.inc'
      include 'eprmat.inc'
      include 'indexf.inc'
      include 'wkspcm.inc'
c
      integer i,j,k,psval,ndim,nev,ierr,mxit,ival,ndone
      double precision zr,zi,zero,bshft,wtolcg,error,errorv
      complex*16 czero,cone,eval,evec,evtmp,x,y,w
      parameter (zero=0.D0,czero=(0.D0,0.D0),cone=(1.D0,0.D0))
      dimension eval(mxstep),evec(mxdim,mxegv),x(mxdim),y(mxdim),
     #          w(mxdim)
c
c#####################################################################
c
c---------------------------------------------------------------------
c     calculate the matrix elements
c---------------------------------------------------------------------
c
      ierr=0
      if (psval.ne.0) then
         ndim=ndimo
         call matrxo(ierr)
         if (idebug.ne.0) then
c                                ** store starting vector **
            open (unit=ludisk,file='stvec.stvx',status='unknown',
     #           access='sequential',form='unformatted')
            write (ludisk) (x(i),i=1,ndim)
            close (unit=ludisk)
c                                ** store matrix elements **
            open (unit=ludisk,file='matrx.mtxx',status='unknown',
     #           access='sequential',form='unformatted')
            write (ludisk) (zdiag(1,i),zdiag(2,i),i=1,ndim)
            write (ludisk) (jzmat(i),i=1,ndim+1) 
            write (ludisk) (izmat(i),zmat(i),i=1,nelim)
            write (ludisk) (kzmat(i),i=1,ndim+1)
            write (ludisk) (izmat(mxel-i+1),zmat(mxel-i+1),i=1,nelre)
            close (unit=ludisk)
         end if
      else
         ndim=ndimd
         call matrxd(ierr)
         if (idebug.ne.0) then
c                                ** store starting vector **
            open (unit=ludisk,file='stvec.stvz',status='unknown',
     #           access='sequential',form='unformatted')
            write (ludisk) (x(i),i=1,ndim)
            close (unit=ludisk)
c                                ** store matrix elements **
            open (unit=ludisk,file='matrx.mtxz',status='unknown',
     #           access='sequential',form='unformatted')
            write (ludisk) (zdiag(1,i),zdiag(2,i),i=1,ndim)
            write (ludisk) (jzmat(i),i=1,ndim+1) 
            write (ludisk) (izmat(i),zmat(i),i=1,nelim)
            write (ludisk) (kzmat(i),i=1,ndim+1)
            write (ludisk) (izmat(mxel-i+1),zmat(mxel-i+1),i=1,nelre)
            close (unit=ludisk)
         end if
      end if
c
      if (ierr.ne.0) then
         write (lulog,1000)
         write (luttyo,1000)
c                  ** make ierr negative for critical error **
         ierr=-abs(ierr)
         return
      end if
c
c---------------------------------------------------------------------
c     perform CG calc to obtain eigenvalues and weighting factors
c---------------------------------------------------------------------
c
      do 20 i=1,mxdim
 20      y(i)=czero
c                          * precondition the matrix for CG
      wtolcg=wtol
      if(psval.eq.1) then
         bshft=shifti+b0
      else if(psval.eq.-1) then
         bshft=shifti-b0
      else
         bshft=shifti
         wtolcg=wtol*0.2d0
      end if
c
      call cscg(x,ndim,nstep+1,cgtol,dcmplx(shiftr,bshft),
     #     y,c2wsp1,cwsp1,cwsp2,cwsp3,cwsp4,cwsp5,w,ndone,error)
c
      if (ndone.le.0) then
         nstep=abs(ndone)
         write (lulog,1010) nstep,error
         if (lulog.ne.luttyo) write (luttyo,1010) nstep,error
      end if
c
      call csval(cwsp1,cwsp2,cwsp3,cwsp4,cwsp5,y,eval,w,ierr)
c
c---------------------------------------------------------------------
c     keep the eigenvalues with
c     (i)  weighting factor greater than wtolcg,
c     (ii) real part is smaller than 20.0 gauss
c         (fastly decaying eigenmodes do not contribute to the signal)
c---------------------------------------------------------------------
c
      nev=0
      do 30 i=1,ngood
        if ((cdabs(w(i)).gt.wtolcg).and.(dreal(eval(i)).lt.20.0d0))
     #                         then
          nev=nev+1
          eval(nev)=eval(i)
          w(nev)=w(i)
        end if
 30   continue
c
c---------------------------------------------------------------------
c     calculate the eigenvectors for nev important eigenvalues
c---------------------------------------------------------------------
c
      if ((idebug.ne.0).and.(ievec.eq.0)) write (ludeb,1020)
      do 100 ival=1,nev
c
         evtmp=eval(ival)
c
         mxit=10
         call csvec(cwsp1,cwsp2,cwsp3,y,evtmp,error,errorv,
     #        cwsp4,cwsp5,iwsp,ndiag,mxit)
c
         do 60 i=1,ndim
            evec(i,ival)=czero
            do 60 j=1,ndiag
               evec(i,ival)=evec(i,ival)+c2wsp1(i,j)*y(j)
 60      continue
c
         if (psval.eq.0) then
            k=1
            do 70 j=1,ndimo
               if (pid(j).eq.1) then
                  evec(j,ival)=-pp(k)*evec(k,ival)
                  k=k+1
               else
                  evec(j,ival)=-pp(k)*evec(k,ival)
     #                         -pp(k+1)*evec(k+1,ival)
                  k=k+2
               end if
 70         continue
         end if

         if (idebug.ne.0) then
            if (ievec.eq.0) then
               write (ludeb,1025) ival,eval(ival),w(ival)
            else
               write (ludeb,1030) ival,eval(ival),w(ival)
               write (ludeb,1040) (j,evec(j,ival),j=1,ndimo)
            end if
         end if
c
 100  continue
c
      return
c
c=====================================================================
c     format statements
c=====================================================================
c
 1000 format(2x,'*** ERROR in generating matrix elements ***')
 1010 format(2x,' CG did not converge in ',i4,' steps : error = ',
     #       g14.7)
 1020 format(/' ** Eigenvalues **'/)
 1025 format(2x,i3,2x,g14.7,1x,g14.7,2x,g14.7,1x,g14.7)
 1030 format(/,' ** Eigenvalue ',i3,' : ',g12.5,1x,g12.5,2x,g12.5,
     #       1x,g12.5,/)
 1040 format(5x,i3,2x,g14.7,1x,g14.7)
c
      end
