c  VERSION 1.1  (NLSPC version)   3/1/94
c----------------------------------------------------------------------
c                         =======================
c                            subroutine PFUN
c                         =======================
c
c Subroutine for interfacing EPRCGF spectral calculations with the
c MINPACK version of the Levenberg-Marquardt nonlinear least squares
c algorithm for fitting experimental spectra. This routine loads the
c necessary parameters into common /eprprm/ and then calls the SIM2D
c subroutine to calculate a spectrum or evaluate a Jacobian as required
c by the calling program.
c
c *** NLSPC only ***
c For evaluation of a series of spectra (which share the eigenvalues and
c eigenvectors), the routine re-evaluates the spectrum using the saved
c eigenvalues and vectors.  Therefore any attempt to fit a series of 
c parameters that affects the eigenvalues is forbidden (e.g. series in
c tilt angle or temperature).
c
c For evaluation of the Jacobian, the routine uses the forward-differences
c approximation to calculate the partial derivative of the spectrum with
c respect to each of the parameters being varied. The routine assumes
c that the function values at the point where the Jacobian is being
c evaluated are contained in fvec and the scaling factor for each spectrum
c in series option is contained in sfac ( common /expdat/ )
c
c In the Jacobian evaluation, the routine makes a special check for 
c variation of the parameters relevent in series option (now gib,lib,
c hwid,mwid).  If line-broadening parameter (gib) is varied, it simply
c re-evaluates the spectrum using the saved time domain data (xspec).
c
c NOTE that in order to follow this strategy, the Jacobian should
c be calculated with respect to any line-broadening parameters before 
c any other parameter is varied (which would change the time domain
c signal in the absence of Gaussian inhomogeneous broadening).
c
c In the future, one might hope to recognize when only the real or 
c imaginary part of the matrix changes, and ultimately to utilize a
c perturbation approach to calculating the Jacobian.
c
c Includes:
c    nlsdim.inc
c    stdio.inc
c    eprprm.inc
c    expdat.inc
c    parcom.inc
c    tdspec.inc
c    wkspcm.inc
c    iterat.inc
c
c Uses
c    sim2d
c    xshft
c    sscale
c---------------------------------------------------------------------- 
c
      subroutine pfun( m,n,x,fvec,fjac,ldfjac,iflag )
      implicit none
      integer m,n,iflag,ldfjac
      double precision x(n),fvec(m),fjac(ldfjac,n)
c
      include 'nlsdim.inc'
      include 'stdio.inc'
      include 'eprprm.inc'
      include 'expdat.inc'
      include 'parcom.inc'
      include 'tdspec.inc'
      include 'wkspcm.inc'
      include 'iterat.inc'
c
      integer i,icalc,ierr,npt1,npt2,isp,ixi,ixs,ixscw,j,k,ld
      double precision rmsdv,shift,resol,ovlap,xtemp,delx
      character dashes*132,jacnam*10
c
      double precision zero
      parameter (zero=0.0D0)
c
c######################################################################
c
c----------------------------------------------------------------------
c    Print parameter values for the current iteration
c----------------------------------------------------------------------
c
      if (iflag.eq.0) then
         ld=12*n+16
         do 1 i=1,ld
            dashes(i:i)='-'
    1       continue
         if (iter.eq.1) then
            if (luout.ne.luttyo) then
               write (luttyo,1006) dashes(:ld)
               write (luttyo,1007) (tag(i),i=1,n)
               write (luttyo,1008) dashes(:ld)
            end if
            write (luout,1006)  dashes(:ld)
            write (luout,1007)  (tag(i),i=1,n)
            write (luout,1008)  dashes(:ld)
         end if
         rmsdv=fnorm/dsqrt(dfloat(nptot))
         if (luout.ne.luttyo) 
     #                write (luttyo,1009) iter,rmsdv,(x(i),i=1,n)
         write (luout,1009) iter,rmsdv,(x(i),i=1,n)
c
c**********************************************************************
c**********************************************************************
c IFLAG=1: Function evaluation
c**********************************************************************
c**********************************************************************
c
      else if (iflag.eq.1) then
c
c                              * replaces xunpak routine
         do 4 j=1,n
            fparm(ixpr(j))=x(j)
 4       continue
c
c     ...Loop over all spectra in a series...
c
         do 5 isp=1,nser
c
            ixs=ixsp(isp)
            ixscw=ixspcw(isp)
c
            icalc=1
            if (isp.eq.1) icalc=2
c
c        ** skip eigenvalue calc. if hwid is the only variables.
            if (seteval.and.(ixpr(n).le.IMWID)) icalc=1
c        ** skip eigenvalue calc. if gib or lib is the only variables.
            if (seteval.and.(ixpr(n).le.ILIB)) icalc=0
c
c----------------------------------------------------------------------
c If desired, find the shift that will give optimal overlap between 
c a spectrum and the data.  This is achieved by comparing cw magnitude
c spectrum with the cw-equivalent spectrum extracted from 2D data,
c which was obtain in data command and saved in cwdata array in common
c block /expdat/.
c----------------------------------------------------------------------
c
            if (dataOK.and.(icalc.ne.0).and.(sishft(isp).ne.0))
     #                       then
c                                            ** cw spectrum
               call setspc( isp,npt1,npt2 )
               fparm(ISHFT)=0.0d0
               iparm(IIEXP)=0
               iparm(IICOMB)=1
               resol=1.0d3/(sstept2(isp)*float(npt2-1))
               npt1=npt2
               npt2=1
c
               call sim2d( icalc,xspectr(ixs),spectr(ixs),
     #                     npt1,npt2,n,iflag,ixi,delx,ierr )
c
               if ((ihltcmd.eq.0).and.(ihltfit.ne.0)) then
                  if (luout.ne.luttyo) write(luttyo,1004)
                  write(luout,1004)
                  iflag=-1
                  return
               else if (ierr.ne.0) then
                  if (luout.ne.luttyo) write(luttyo,1003)
                  write(luout,1003)
                  iflag=-1
                  return
               end if
c
               call xshft(cwdata(ixscw),spectr(ixs),npt1,cwsp1,cwsp2,
     #                    shift,ovlap)
               sshft(isp)=resol*shift
c                        * no need for eigenvalue calculation next
               icalc=1
            end if
c
c----------------------------------------------------------------------
c        Calculate 2D spectrum with desired shift term (shft).
c----------------------------------------------------------------------
c
            call setspc( isp,npt1,npt2 )
            call sim2d( icalc,xspectr(ixs),spectr(ixs),
     #                  npt1,npt2,n,iflag,ixi,delx,ierr )
c
            if ((ihltcmd.eq.0).and.(ihltfit.ne.0)) then
               if (luout.ne.luttyo) write(luttyo,1004)
               write(luout,1004)
               iflag=-1
               return
            else if (ierr.ne.0) then
               if (luout.ne.luttyo) write(luttyo,1003)
               write(luout,1003)
               iflag=-1
               return
            end if
c
c               ... End of loop over spectra in a series
c
 5       continue
c
c----------------------------------------------------------------------
c Calculate the scale factor(s) for each spectrum in the series,
c according to the dependencies (idepnd) that user specified in data
c command.  The subroutine returns the scaled spectra and the scale
c factors (sfac in common block /expdat/).
c----------------------------------------------------------------------
c
         if (dataOK) call sscale(spectr)
c
c----------------------------------------------------------------------
c Calculate the difference between the experimental and scaled
c calculated spectra and store the result in fvec.
c----------------------------------------------------------------------
c
         do 20 j=1,m
            fvec(j)=spectr(j)-data(j)
 20      continue
c
         if (idebug.ne.0) then
            write (ludeb,2010)
            do 21 j=1,nser
               write (ludeb,2020) j,sfac(j),sshft(j)
 21         continue
         end if
c
c        ** Update X vector for the case where the real part of
c           the eigenvalues are negative.
         do 6 j=1,n
            x(j)=fparm(ixpr(j))
 6       continue
c
c######################################################################
c  *** OBTAIN 2D PLOTTING PACKAGE FOR PFUN ***
c        Plot the current calculation
c           do 101 i=1,nser
c              call fstpltpdt( data(ixsp(i)), fvec(ixsp(i)), 
c     #             sbi(i)+shft(i), sdb(i), ndata(i), i )
c 101       continue
c
c######################################################################
c
c**********************************************************************
c**********************************************************************
c IFLAG >= 2: Jacobian evaluation by forward-difference approximation:
c             fjac = (f(x0+dx)-f(x0))/dx
c
c          where f(x0) = fvec+data, and dx is assumed to be positive
c**********************************************************************
c**********************************************************************
c
      else if (iflag.ge.2) then
c
c----------------------------------------------------------------------
c Loop over all parameters, introducing the forward-difference step
c into each parameter
c----------------------------------------------------------------------
c
         do 30 i=1,n
            xtemp=x(i)
            x(i)=x(i)+xfdstp(i)
            ixi=ixpr(i)
            delx=xfdstp(i)
c                              * replaces xunpak routine
            do 24 j=1,n
              fparm(ixpr(j))=x(j)
 24         continue
c
c        ...Loop over all spectra in a series...
c
            do 27 isp=1,nspc
c
               ixs=ixsp(isp)
c
c          ... Store function value before forward-differences step
c
               do 25 j=1,ndata(isp)
                  k=ixs+j-1
                  fjac(k,i)=-(fvec(k)+data(k))
 25            continue
c
	       icalc=1
               if ((isp.eq.1).and.(ixi.gt.IHWID)) icalc=2
               if (ixi.le.ILIB) icalc=0
c
               call setspc( isp,npt1,npt2 )
               call sim2d( icalc,xspectr(ixs),spectr(ixs),
     #                     npt1,npt2,n,iflag,ixi,delx,ierr )
c
               if ((ihltcmd.eq.0).and.(ihltfit.ne.0)) then
                  if (luout.ne.luttyo) write(luttyo,1004)
                  write(luout,1004)
                  iflag=-1
                  return
               else if (ierr.ne.0) then
                  if (luout.ne.luttyo) write(luttyo,1003)
                  write(luout,1003)
                  iflag=-1
                  return
               end if
c
c     Add in each calculated forward-differences spectrum
c
               do 26 j=1,ndata(isp)
                  k=ixs+j-1
                  fjac(k,i)=fjac(k,i)+sfac(isp)*spectr(k)
 26            continue
c
c                  ... End of loop over spectra in a series
c
 27         continue
c
c----------------------------------------------------------------------
c Use the step size to calculate the partial derivative w.r.t. x(i)
c for the current spectrum in the series
c----------------------------------------------------------------------
c
            do 28 j=1,m
               fjac(j,i)=fjac(j,i)/delx
 28         continue
c
            x(i)=xtemp
c                     *** end of loop over parameters
 30      continue
c
c----------------------------------------------------------------------
c     Optional output of Jacobian matrix for this iteration
c----------------------------------------------------------------------  
c
         if (iflag.eq.3) then
            write(jacnam,2000) iter
            open(unit=ludisk,file=jacnam,status='unknown',
     #           access='sequential',form='formatted')
            do 40 i=1,m
               write (ludisk,2003) data(i),fvec(i),(fjac(i,j),j=1,n)
 40         continue
            close(ludisk)
         end if
c
      end if
c
      return
c
c ### format statements ##############################################
c
 1003 format(/20x,'*** Error: fit or search procedure terminating',
     #       ' ***'/)
 1004 format(/20x,'*** Fit halted by user ***')
 1006 format(/a)
 1007 format('Iter',5x,'RmsDv',10(6x,a))
 1008 format(a)
 1009 format(i4,4x,g10.4,2x,10(g12.6))
 2000 format('nlsjac.',i3.3)
 2003 format(10(g12.5,1x))
 2010 format(/' ** ',6x,'Spectrum',5x,'Scale Factor',5x,
     #       'Shift Factor **')
 2020 format(13x,i1,7x,g14.7,3x,g14.7)
c
      end

c----------------------------------------------------------------------
c                    =========================
c                       subroutine SETSPC
c                    =========================
c
c  Sets the series-dependent parameters in the fparm and iparm arrays.
c  These parameters are not allowed to change the eigenvalues.
c
c  Specifically, the parameters allowed in series option are
c
c    Integer :  iexp, icomb, npt1, npt2
c    Floating:  init1, stept1, init2, stept2, tfix, shft
c
c----------------------------------------------------------------------
c
      subroutine setspc( ispc,npt1,npt2 )
      implicit none
      integer ispc,npt1,npt2
c
      include 'nlsdim.inc'
      include 'eprprm.inc'
      include 'expdat.inc'
      include 'parcom.inc'
c
      if (ispc.le.0 .or. ispc.gt.nser) return
c
      fparm(IINIT1) = sinit1(ispc)
      fparm(ISTEPT1) = sstept1(ispc)
      fparm(IINIT2) = sinit2(ispc)
      fparm(ISTEPT2) = sstept2(ispc)
      fparm(ITFIX) = stfix(ispc)
      fparm(ISHFT) = sshft(ispc)
c
      iparm(IIEXP) = siexp(ispc)
      iparm(IICOMB) = sicomb(ispc)
      npt1 = snpt1(ispc)
      npt2 = snpt2(ispc)
c
      return
      end
