c  VERSION 1.0  (NLSP version)   7/23/93
c----------------------------------------------------------------------
c                    =========================
c                       subroutine SRCHC
c                    =========================
c
c   search  <parameter> { tol <tol> step <step>
c                         min <smn> max <smx> start <val> }
c
c    tol     : Convergence tolerance for fitting parameter
c    step    : Size of first step to take away from initial parameter value  
c    smn,smx : minimum and maximum that defines the search range
c    val     : starting value of the Brent's search procedure
c              (Note that the bracketing procedure is not performed
c               if this option is used.  Therefore the used have to
c               make sure that there exists a minimum chi-squared within
c               the range.)
c
c----------------------------------------------------------------------
      subroutine srchc( line )
      implicit none
      character line*80
c
      include 'nlsdim.inc'
      include 'eprprm.inc'
      include 'parcom.inc'
      include 'lpnam.inc'
      include 'stdio.inc'
c
      integer ixp1p,i,itmp,ierr,lth,ibrckt
      double precision ax,bx,cx,fa,fb,fc,fval,xmin,dummy
      character token*30,prmID*6
c
      integer nkeywd
      parameter(nkeywd=5)
c
      character*8 keywrd(nkeywd)
      data keywrd / 'TOL','STEP','MIN','MAX','START'/
c
      logical ftoken,prmsOK,tcheck
      integer ipfind
      double precision brent,p1pfun
      external ftoken,prmsOK,tcheck,ipfind,brent,p1pfun
c
c######################################################################
c
      if (.not.setbas) then
         write (luout,1200)
         if (luout.ne.luttyo) write (luttyo,1200)
         return
      end if
c
c  -- Check whether any starting parameters have been input before 
c     proceeding with the FIT command
c
      if (.not.prmsOK(luttyo) ) then
        write(luttyo,1040)
        return
      end if
c
      ierr=0
      ibrckt=0
c
c----------------------------------------------------------------------
c Get the name of the parameter
c----------------------------------------------------------------------
      call gettkn(line,token,lth)
      lth=min(lth,6)
      call touppr(token,lth)
c
      ixp1p=ipfind(token,lth)
c
c----------------------------------------------------------------------
c     Check whether parameter may be varied 
c----------------------------------------------------------------------
      if (ixp1p.eq.0 .or. ixp1p.gt.nfprm) then
         write(luttyo,1002) token(:lth)
         return
      end if
c
      if (ixp1p.lt.-100) then 
        prmID=alias2( -99-(IGXX+ixp1p) )
      else if (ixp1p.lt.0) then
        prmID=alias1( 1-(IGXX+ixp1p) )
      else
        prmID=parnam(ixp1p)
      end if
c
c  --- Check whether proper symmetry has been specified for
c      tensor components 
c
      if (.not.tcheck(ixp1p,prmID,luttyo)) return
      ixp1p=iabs( mod(ixp1p,100) )
c
c Set default search range
c
      smn=fparm(ixp1p)-1.0d0
      smx=fparm(ixp1p)+1.0d0
c
      if (ixp1p.eq.IC20) smn=1.0d-2
c
c----------------------------------------------------------------------
c  Look for a keyword
c----------------------------------------------------------------------
c
   14 call gettkn(line,token,lth)
      lth=min(lth,8)
c
c------------------------------------------------
c************************************************
c No more keywords: call the line search routine
c************************************************
c------------------------------------------------
      if (lth.eq.0) then
c
         seteval=.false.
c
c Set two initial parameter values for the parameter to be searched
c
         ax=fparm(ixp1p)
         bx=fparm(ixp1p)+sstep
c
         if (smn.gt.ax) smn=ax-1.0d0
         if (smx.lt.ax) smx=ax+1.0d0
c
c  Swap indices for the search parameter into the first elements 
c  of the parameter for the NLS x-vector
c  (This is so x can be used by the pfun routine as in a NLS procedure)
c
         fval=fparm(ixpr(1))
         itmp=ixpr(1)
c
         xmin=fparm(ixp1p)
         ixpr(1)=ixp1p
c
         if (ibrckt.eq.0) then
            write (luout,1100) smn,ax,smx
            if (luout.ne.luttyo) write (luttyo,1100) smn,ax,smx
            call mnbrak(ax,bx,cx,fa,fb,fc,p1pfun,smn,smx,ierr)
            if (ierr.lt.0) return
            write (luout,1105) ax,bx,cx,fa,fb,fc
            if (luout.ne.luttyo) write (luttyo,1105) ax,bx,cx,fa,fb,fc
         else
            ax=smn
            bx=stval
            cx=smx
            write (luout,1100) smn,stval,smx
            if (luout.ne.luttyo) write (luttyo,1100) smn,stval,smx
         end if
c
         if (ierr.eq.0) then
            dummy=brent(ax,bx,cx,p1pfun,stol,xmin,prmID,ierr)
            if (ierr.lt.0) return
         else
            write (luout,1110)
            if (luout.ne.luttyo) write (luttyo,1110)
            if (ax.gt.bx) then
               write (luout,1120) cx,ax,fc,fa
               if (luout.ne.luttyo) write (luttyo,1120) cx,ax,fc,fa
            else
               write (luout,1120) ax,cx,fa,fc
               if (luout.ne.luttyo) write (luttyo,1120) ax,cx,fa,fc
            end if
         end if
c                            ** put the parameters back
         fparm(ixp1p)=xmin
         if (ixpr(1).ne.itmp) then
            ixpr(1)=itmp
            fparm(itmp)=fval
         end if
         return
      end if
c
c------------------------------
c Check keyword list
c------------------------------
      call touppr(token,lth)
      do 15 i=1,nkeywd
         if (token(:lth).eq.keywrd(i)(:lth)) goto 16
 15   continue
c                                        *** Unrecognized keyword
      write (luttyo,1000) token(:lth)
      return
c
c----------------------------------------------------------------------
c  Keyword found: for keywords requiring an argument, convert 
c  next token and assign appropriate value
c----------------------------------------------------------------------
 16   call gettkn(line,token,lth)
c                                        *** No value given
      if (lth.eq.0) then
         write(luttyo,1003) keywrd(i)
         return
      end if
c
      if (ftoken(token,lth,fval)) then
c                                          *** TOL keyword
         if (i.eq.1) then
            stol=fval
c                                          *** STEP keyword
         else if (i.eq.2) then 
            sstep=fval
c                                          *** MIN keyword
         else if (i.eq.3) then
            smn=fval
c                                          *** MIN keyword
         else if (i.eq.4) then
            smx=fval
c                                          *** START keyword
         else if (i.eq.5) then
            ibrckt=1
            stval=fval
c                                      *** Illegal numeric value
         end if
      else
         write(luttyo,1001) token(:lth)
      end if
      go to 14
c
c ##### Format statements ########################################
c
 1000 format('*** Unrecognized SEARCH keyword: ''',a,''' ***')
 1001 format('*** Numeric value expected: ''',a,''' ***')
 1002 format('*** ''',a,''' is not a variable parameter ***')
 1003 format('*** No value given for ''',a,''' ***')
 1040 format('*** Initial parameter values are required before',
     #      ' a fit ***')
 1100 format(/4x,'Initial Range  : ',g13.6,2x,g13.6,2x,g13.6)
 1105 format(4x,'Search Bracket : ',g13.6,2x,g13.6,2x,g13.6,
     #      /4x,'Rms Deviation  : ',g13.6,2x,g13.6,2x,g13.6)
 1110 format('*** No minimum found within the valid range ***')
 1120 format(4x,'Last Bracket   : ',g13.6,2x,g13.6,
     #      /4x,'Rms Deviation  : ',g13.6,2x,g13.6)
 1200 format(/,'** Basis Index set is not properly set **')
      end
